import { Extension } from '@codemirror/state';
import { EditorView } from '@codemirror/view';
import { Accessor } from 'solid-js';
export declare type CompartmentReconfigurationCallback = (extension: Extension) => void;
/**
 * Creates a compartment extension for the given CodeMirror EditorView.
 *
 * Extension compartments can be used to make a configuration dynamic.
 * By wrapping part of your configuration in a compartment, you can later replace that part through a transaction.
 *
 * See {@link https://codemirror.net/examples/reconfigure/} for use cases and examples of `Compartments`.
 * Check out {@link https://codemirror.net/docs/ref/#state.Compartment} for more details about Compartment API.
 *
 * @param extension The extension to wrap in a compartment.
 * @param view The CodeMirror EditorView
 */
export declare function createCompartmentExtension(extension: Accessor<Extension | undefined> | Extension, view: Accessor<EditorView | undefined>): CompartmentReconfigurationCallback;
